package aood.calcsheet.dbjp;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

/**
 * Class for calculating minimum and maximum values in a sheet.
 * Executable class, with main(String[]) method, should be run
 * in following way:
 * java CalcSheet [input_file] [output_file]
 * 
 * @author Daniel Badtke, Jacek Pospychała
 *
 */
public class CalcSheet {
	/**
	 * Number of row with minimum values.
	 */
	public static final int MIN_VALUES_ROW = 0;
	
	/**
	 * Number of row with maximum values.
	 */
	public static final int MAX_VALUES_ROW = 1;

	/**
	 * Calculate minimum and maximum values in an input sheet
	 * columns. Every column of input sheet, has corresponding
	 * column in result, containing minimum value in first 
	 * row and maximum in second row.
	 * 
	 * @param inputSheet sheet with rows to be calculated.
	 * @return result sheet with minimum and maximum values.
	 */
	public static Sheet calculateMinMax(Sheet inputSheet) {
		Sheet result = new Sheet(inputSheet.getWidth(), 2);
		int colNum;
		List<Long> column;
		
		for(colNum = 0; colNum < inputSheet.getWidth(); colNum++) {
			column = inputSheet.getColumn(colNum);
			
			Long colMin = Collections.min(column);
			Long colMax = Collections.max(column);
			
			result.set(colNum, MIN_VALUES_ROW, colMin);
			result.set(colNum, MAX_VALUES_ROW, colMax);
		}
		
		return result;
	}
	
	/**
	 * Executable method. Command line arguments should be:
	 * input file name, output file name.
	 * Input file has the input sheet to be evaluated.
	 * Result will be written to the output file.<br> 
	 * 
	 * If less than two
	 * arguments are given, application exits with appropriate
	 * message. In other way application will read the file
	 * and save result calculated as defined 
	 * in {@link #calculateMinMax(Sheet)}
	 * 
	 * @param args Command line arguments.
	 */
	public static void main(String[] args) {
		Sheet input = new Sheet();
		Sheet result;
		
		if(args.length < 2) {
			System.err.println("Usage: CalcSheet in.txt out.txt");
			System.exit(1);
		}
		
		try {
			input.load(args[0]);
			result = CalcSheet.calculateMinMax(input);
			result.save(args[1]);
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
