package pl.put.poznan.metrics.calculators.ck;

import java.util.Set;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;
import poznan.students.utils.CalculatorCache;

public class CouplingBetweenObject extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na CBO - Coupling
	 * Between Object
	 * 
	 */
	public CouplingBetweenObject() {
		super(CBO);
		// TODO Auto-generated constructor stub
	}

	/**
	 * Obliczenie metryki CBO. Metrka jest obliczana na poziomie "Typw"
	 * 
	 * @param AbstractMetricSource -
	 *            typ na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>TYPE</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		if (source.getLevel() != TYPE)
			throw new InvalidSourceException(
					"InheritanceDepth only applicable to types");

		IType iType = (IType) source.getJavaElement();

		try {

			Util.countCoupling(iType.getJavaProject());

			int result = 0;

			CalculatorCache calcCache = Util.getProjectCache(iType
					.getJavaProject().getElementName());

			Object oSet = calcCache.getCouplingMap().get(
					iType.getFullyQualifiedName());

			if ( null == oSet){
				Logger.logMessage("CBO result = " + result);
				source.setValue(new Metric(CBO, 0));
				return;
			}
			
			result = ((Set) oSet).size();

			Logger.logMessage("CBO result = " + result);
			source.setValue(new Metric(CBO, result));

		} catch (JavaModelException e) {
			source.setValue(new Metric(CBO, 0));
			Logger.logError("CBO exception", e);
			Logger.logMessage("CBO result = " + 0);
		}

	}

}
