package pl.put.poznan.metrics.calculators.ck;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;
import net.sourceforge.metrics.core.sources.TypeMetrics;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;

public class ResponseForAClass extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na RFC - Response for
	 * a class
	 * 
	 */
	public ResponseForAClass() {
		super(RFC);
	}

	/**
	 * Obliczenie metryki RFC. Metrka jest obliczana na poziomie "Typw"
	 * 
	 * @param AbstractMetricSource -
	 *            typ na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>TYPE</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		if (source.getLevel() != TYPE)
			throw new InvalidSourceException(
					"InheritanceDepth only applicable to types");

		IType iType = (IType) source.getJavaElement();
		TypeMetrics tm = (TypeMetrics) source;
		ITypeHierarchy hierarchy = tm.getHierarchy();

		try {

			int result = 0;

			result = getSubclassMethodCount(iType, hierarchy);

			Logger.logMessage("RFC result = " + result);
			source.setValue(new Metric(RFC, result));

		} catch (JavaModelException e) {
			source.setValue(new Metric(RFC, 0));
			Logger.logError("RFC exception", e);
			Logger.logMessage("RFC result = " + 0);
		}

	}
	
	/**
	 * Oblicza ilo metod w typach pochodnych od tego podanego w parametrze
	 * @param itype - typ
	 * @param hierarchy - hierarchia typw
	 * @return int - liczba metod w typach pochodnych
	 * @throws JavaModelException
	 */
	public int getSubclassMethodCount(IType itype, ITypeHierarchy hierarchy)
			throws JavaModelException {

		int count = 0;

		count = Util.getMethods(itype, Flags.AccPublic).size();

		IType[] subs = hierarchy.getSubclasses(itype);

		for (int i = 0; i < subs.length; i++) {
			count += getSubclassMethodCount(subs[i], hierarchy);
		}

		return count;
	}
}
