package pl.put.poznan.metrics.calculators.mood;

import java.util.Iterator;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;

public class AttributeHidingFactor extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na AHF - Attribute
	 * Hiding Factor
	 * 
	 */
	public AttributeHidingFactor() {
		super(AHF);
		Logger.logMessage("AHF - konstruktor");
	}

	/**
	 * Obliczenie metryki AHF. Metrka jest obliczana na poziomie Projektu
	 * 
	 * @param AbstractMetricSource -
	 *            projekt na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>PROJECT</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double hideAttributes = 0;
		double allAttributes = 0;

		if (source.getLevel() != PROJECT)
			throw new InvalidSourceException(
					"Attribute Hiding Factor only applicable to project");

		IJavaProject iProject = (IJavaProject) source.getJavaElement();

		try {

			Iterator iter = Util.getAllTypes(iProject).iterator();

			while (iter.hasNext()) {
				IType iType = (IType) iter.next();

				allAttributes += iType.getFields().length;

				hideAttributes += AttributeHidingFactor
						.getCountHiddenAttributes(iType);

			}

			double result = 100d;

			Logger.logMessage("AHF all = " + allAttributes);
			Logger.logMessage("AHF hide = " + hideAttributes);

			if (allAttributes != 0) {
				result = (long) ((hideAttributes / allAttributes) * 100);
			}

			Logger.logMessage("AHF result = " + result);
			source.setValue(new Metric(AHF, result));

		} catch (JavaModelException e) {
			source.setValue(new Metric(AHF, 0));
			Logger.logError("AHF exception", e);
			Logger.logMessage("AHF result = " + 0);
		}

	}

	/**
	 * Liczy uklryte atrybuty, czyli atrybuty private lub protected
	 * 
	 * @param iType - typ
	 * @return int - liczba ukrytych atrybutw
	 * @throws JavaModelException
	 */
	public static int getCountHiddenAttributes(IType iType)
			throws JavaModelException {

		int hiddenAttributes = 0;

		IField[] iFileds = iType.getFields();

		for (int index = 0; index < iFileds.length; index++) {

			if (((iFileds[index].getFlags() & Flags.AccPrivate) != 0)
					|| ((iFileds[index].getFlags() & Flags.AccProtected) != 0)) {

				hiddenAttributes++;

			}
		}

		return hiddenAttributes;

	}

}
