package pl.put.poznan.metrics.calculators.mood;

import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;


public class AttributeHidingFactorPerClass extends Calculator implements
		ExtensionsConstants {


	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na AHF_PER_CLASS - Attribute Hiding Factor
	 * na poziomie klasy
	 * 
	 */
	public AttributeHidingFactorPerClass() {
		super(AHF_PER_CLASS);
		Logger.logMessage("AHF_PER_CLASS - konstruktor");
		//kalkulator uruchamiany jako pierwszy czysci tymczasowe dane
		
	}

	/**
	 * Obliczenie metryki AHF_PER_CLASS. Metrka jest obliczana na poziomie Typw
	 * 
	 * @param AbstractMetricSource -
	 *            typ na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>TYPE</code>
	 */
	public void calculate(AbstractMetricSource source)
		throws InvalidSourceException {

		double hideAttributes = 0;
		double allAttributes = 0;

		if (source.getLevel() != TYPE) 
			throw new InvalidSourceException("Attribute Hiding Factor only applicable to types");

		IType iType = (IType) source.getJavaElement();
		
		try {
		
			allAttributes += iType.getFields().length;
			hideAttributes += AttributeHidingFactor.getCountHiddenAttributes(iType);

			double result = 100d;

			Logger.logMessage("AHF_PER_CLASS all = " + allAttributes);
			Logger.logMessage("AHF_PER_CLASS hide = " + hideAttributes);

			if ( allAttributes != 0 ){
				result = (long)((hideAttributes / allAttributes) * 100);
			}

			Logger.logMessage("AHF_PER_CLASS result = " + result);
			source.setValue(new Metric(AHF_PER_CLASS,result));
			
		} catch (JavaModelException e) {
			source.setValue(new Metric(AHF_PER_CLASS,0));
			Logger.logMessage("AHF_PER_CLASS result = " + 0);
			Logger.logError("AHF_PER_CLASS error",e);
			
		}
		
	}	

//	oblicza na poziomie compilation unit ( ale nie pobiera interfejsow :( ) 
	/*public void calculate(AbstractMetricSource source)
		throws InvalidSourceException {

		double hideAttributes = 0;
		double allAttributes = 0;

		if (source.getLevel() != COMPILATIONUNIT) 
			throw new InvalidSourceException("Attribute Hiding Factor only applicable to compilation unit");

		ICompilationUnit iCompilationUnit = (ICompilationUnit) source.getJavaElement();
		
		try {
		
			IType[] iTypes = iCompilationUnit.getTypes();

			for (int iii = 0; iii < iTypes.length; iii++) {
				
				allAttributes += iTypes[iii].getFields().length;
				hideAttributes += AttributeHidingFactor.getCountHiddenAttributes(iTypes[iii]);
				
			}

			double result = 100d;

			Logger.logMessage("all = " + allAttributes);
			Logger.logMessage("hide = " + hideAttributes);

			if ( allAttributes != 0 ){
				result = (long)((hideAttributes / allAttributes) * 100);
			}

			Logger.logMessage("AHF_PER_CLASS result = " + result);
			source.setValue(new Metric(AHF_PER_CLASS,result));
			
		} catch (JavaModelException e) {
			source.setValue(new Metric(AHF_PER_CLASS,0));
		}
		

	}*/	

	
}
