package pl.put.poznan.metrics.calculators.mood;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

public class AttributeInheritanceFactor extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na AIF - Attribute Inheritance Factor
	 * 
	 * 
	 */
	public AttributeInheritanceFactor() {
		super(AIF);
		Logger.logMessage("AIF - konstruktor");

	}

	/**
	 * Obliczenie metryki AIF. Metrka jest obliczana na poziomie Projektu
	 * 
	 * @param AbstractMetricSource -
	 *            projekt na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>PROJECT</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double inheritedAttributes = 0;
		double allAttributes = 0;

		if (source.getLevel() != PROJECT)
			throw new InvalidSourceException(
					"Attribute Inheritance Factor only applicable to project");

		IJavaProject iProject = (IJavaProject) source.getJavaElement();

		try {

			IPackageFragment[] iPackagesRoot = iProject.getPackageFragments();

			for (int i = 0; i < iPackagesRoot.length; i++) {

				ICompilationUnit[] iCompilationUnits = iPackagesRoot[i]
						.getCompilationUnits();

				for (int j = 0; j < iCompilationUnits.length; j++) {

					IType[] iTypes = iCompilationUnits[j].getAllTypes();

					for (int k = 0; k < iTypes.length; k++) {

						int inhTemp = 0;

						Set superClassesFields = getSuperClassesFieldsSignatures(
								iTypes[k], iProject);

						// all attributes in this class
						allAttributes += iTypes[k].getFields().length;

						// inheritted attributes
						inhTemp = getCountInherittedFields(iTypes[k],
								superClassesFields);
						inheritedAttributes += inhTemp;

						// all attreibutes plus inherited attriubutes
						allAttributes += inhTemp;

					}

				}
			}

			double result = 0d;

			Logger.logMessage("AIF, allAttributes = " + allAttributes);
			Logger.logMessage("AIF, inheritedAttributes = "
					+ inheritedAttributes);

			if (allAttributes != 0) {
				result = (long) ((inheritedAttributes / allAttributes) * 100);
			}

			Logger.logMessage("AIF, result = " + result);

			source.setValue(new Metric(AIF, result));

		} catch (JavaModelException e) {
			Logger.logError("AIF exception", e);
			source.setValue(new Metric(AIF, 0));
		}

	}
	/**
	 * Pobiera sygnatury metod z nadtypw
	 * @param iType
	 * @param proj
	 * @return Set - zbir sygnatw
	 * @throws JavaModelException
	 */
	public static Set getSuperClassesFieldsSignatures(IType iType,
			IJavaProject proj) throws JavaModelException {

		// TypeMetrics tm = (TypeMetrics)iType;

		Set superClassesFields = new HashSet();

		ITypeHierarchy hierarchy = iType.newTypeHierarchy(proj, null);

		IType[] supeClasses = hierarchy.getAllSuperclasses(iType);

		for (int i = 0; i < supeClasses.length; i++) {

			IField[] fields = supeClasses[i].getFields();

			for (int j = 0; j < fields.length; j++) {
				if ((fields[j].getFlags() & Flags.AccPrivate) == 0) {

					superClassesFields.add(fields[j].getElementName());

				}
			}
		}

		return superClassesFields;

	}
	
	/**
	 * Liczy ilo odziedziczonych pl
	 * @param iType
	 * @param superClassesFields
	 * @return int - liczba odziedziczonych atrybut
	 * @throws JavaModelException
	 */
	public static int getCountInherittedFields(IType iType,
			Set superClassesFields) throws JavaModelException {

		int inheritedAttributes = superClassesFields.size();

		IField[] iFileds = iType.getFields();

		for (int index = 0; index < iFileds.length; index++) {

			if (superClassesFields.contains(iFileds[index].getElementName())) {
				// attribut overide att from parent class
				inheritedAttributes--;
			}
		}

		return inheritedAttributes;
	}
}
