package pl.put.poznan.metrics.calculators.mood;

import java.util.Set;

import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;
import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;

public class AttributeInheritanceFactorPerClass extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na AIF_PER_CLASS - Attribute Inheritance Factor
	 * na poziomie pojedynczego typu
	 * 
	 */
	public AttributeInheritanceFactorPerClass() {
		super(AIF_PER_CLASS);
		Logger.logMessage("AIF_PER_CLASS - konstruktor");
	}

	/**
	 * Obliczenie metryki AIF_PER_CLASS. Metrka jest obliczana na poziomie typw
	 * 
	 * @param AbstractMetricSource -
	 *            typ na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>TYPE</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {
		double inheritedAttributes = 0;
		double allAttributes = 0;

		if (source.getLevel() != TYPE)
			throw new InvalidSourceException(
					"Attribute Inheritance Factor only applicable to types");

		IType iType = (IType) source.getJavaElement();

		try {

			Set superClassesFields = AttributeInheritanceFactor
					.getSuperClassesFieldsSignatures(iType, iType
							.getJavaProject());

			// all attributes in this class
			allAttributes = iType.getFields().length;

			// inheritted attributes
			inheritedAttributes = AttributeInheritanceFactor
					.getCountInherittedFields(iType, superClassesFields);

			// all attreibutes plus inherited attriubutes
			allAttributes += inheritedAttributes;

			double result = 0d;

			Logger.logMessage("AIF_CL, allAttributes = " + allAttributes);
			Logger.logMessage("AIF_CL, inheritedAttributes = "
					+ inheritedAttributes);

			if (allAttributes != 0) {
				result = (long) ((inheritedAttributes / allAttributes) * 100);
			}

			Logger.logMessage("AIF_CL, result = " + result);

			source.setValue(new Metric(AIF_PER_CLASS, result));

		} catch (JavaModelException e) {
			Logger.logError("AIF_CL exception", e);
			source.setValue(new Metric(AIF_PER_CLASS, 0));
		}
	}

	// pre compilation unit version
	/*
	 * public void calculate(AbstractMetricSource source) throws
	 * InvalidSourceException { double inheritedAttributes = 0; double
	 * allAttributes = 0;
	 * 
	 * if (source.getLevel() != COMPILATIONUNIT) throw new
	 * InvalidSourceException("Attribute Inheritance Factor only applicable to
	 * types");
	 * 
	 * ICompilationUnit iCompilation = (ICompilationUnit)
	 * source.getJavaElement();
	 * 
	 * 
	 * try {
	 * 
	 * IType [] iTypes = iCompilation.getTypes();
	 * 
	 * for (int i = 0; i < iTypes.length; i++) {
	 * 
	 * List superClassesFields =
	 * AttributeInheritanceFactor.getSuperClassesFieldsSignatures(iTypes[i],
	 * iTypes[i].getJavaProject());
	 * 
	 * //all attributes in this class
	 * allAttributes+=iTypes[i].getFields().length;
	 * 
	 * //inheritted attributes inheritedAttributes+=
	 * AttributeInheritanceFactor.getCountInherittedFields(iTypes[i],superClassesFields);
	 * 
	 * //all attreibutes plus inherited attriubutes
	 * allAttributes+=inheritedAttributes;
	 *  }
	 * 
	 * double result = 100d;
	 * 
	 * Logger.logMessage("AIF, allAttributes = " + allAttributes);
	 * Logger.logMessage("AIF, inheritedAttributes = " + inheritedAttributes);
	 * 
	 * if ( allAttributes != 0 ){ result = (long)((inheritedAttributes /
	 * allAttributes) * 100); }
	 * 
	 * Logger.logMessage("AIF, result = " + result);
	 * 
	 * source.setValue(new Metric(AIF,result));
	 * 
	 *  } catch (JavaModelException e) { Logger.logError("AIF exception", e);
	 * source.setValue(new Metric(AIF,0)); } }
	 */

}
