package pl.put.poznan.metrics.calculators.mood;

import java.util.Iterator;
import java.util.Set;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Log;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;
import poznan.students.utils.CalculatorCache;

public class CouplingFactor extends Calculator implements ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na CF - Coupling
	 * Factor
	 * 
	 */
	public CouplingFactor() {
		super(CF);
		Logger.logMessage("CF_PER_CLASS - konstruktor");
	}

	/**
	 * Obliczenie metryki CF. Metrka jest obliczana na poziomie Projektu
	 * 
	 * @param AbstractMetricSource -
	 *            projekt na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>PROJECT</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double count = 0;
		double numberOfClasses = 0;

		double dc = 0;

		if (source.getLevel() != PROJECT)
			throw new InvalidSourceException(
					"Coupling Factor only applicable to project");

		Logger.logMessage("CF zaczynam");

		IJavaProject iProject = (IJavaProject) source.getJavaElement();

		try {

			Iterator iter = Util.getAllTypes(iProject).iterator();

			Util.countCoupling(iProject);

			while (iter.hasNext()) {
				IType iType = (IType) iter.next();

				// Logger.logMessage("CF, working on type = " +
				// iType.getElementName() + ", licznik = " + currentTypeNumber +
				// "/" + totalCount);

				// search(iProject,iType);

				numberOfClasses++;

				/*
				 * count+= getCountCoupledClasses(iType, iProject);
				 * numberOfClasses++;
				 * 
				 * dc += Util.countDescendants(iType, iProject);
				 */

				dc += Util.countDescendants(iType, iProject);

			}

			CalculatorCache cache = Util.getProjectCache(iProject
					.getElementName());

			Iterator keysIter = cache.getCouplingMap().keySet().iterator();

			while (keysIter.hasNext()) {
				String key = (String) keysIter.next();
				Logger.logMessage("CF key = " + key);
				count += ((Set) cache.getCouplingMap().get(key)).size();
			}

			Logger.logMessage("CF koniec");
			Logger.logMessage("CF count = " + count);
			Logger.logMessage("CF numberOfClasses = " + numberOfClasses);
			Logger.logMessage("CF dc = " + dc);
			double result = 0d;

			if (0 != ((numberOfClasses * numberOfClasses) - numberOfClasses - 2 * dc)) {
				result = (count / ((numberOfClasses * numberOfClasses)
						- numberOfClasses - 2 * dc)) * 100;
			}

			Logger.logMessage("CF result = " + result);
			source.setValue(new Metric(CF, result));

		} catch (JavaModelException e) {
			source.setValue(new Metric(CF, 0));
			Logger.logError("CF exception", e);
			Logger.logMessage("CF result = " + 0);
		}

	}

	/*
	 * public int getCountCoupledClasses(IType currentClass, IJavaProject
	 * project) throws JavaModelException{
	 * 
	 * int coupledClasses = 0;
	 * 
	 * Iterator iter = Util.getAllTypes(project).iterator();
	 * 
	 * while ( iter.hasNext()){ IType iType = (IType) iter.next();
	 * 
	 * if ((!currentClass.equals(iType)) &&
	 * (!isSuperClass(currentClass,iType,project))){ if (search(currentClass,
	 * iType) ){ Logger.logMessage("CF, klasa"+ iType.getElementName() +",
	 * powiazana z " + currentClass.getElementName()); coupledClasses++; } } }
	 * 
	 * return coupledClasses; }
	 */

	/**
	 * @param iProject - projekt w ktorym zostanie wyszukana referenca do podanego typu
	 * @param searchFor - szukany typ
	 * @return boolean - okrela czy tym zosta znaleziony
	 */
	public boolean search(IJavaProject iProject, IType searchFor) {
		SearchPattern pattern = SearchPattern.createPattern(searchFor,
				IJavaSearchConstants.REFERENCES);
		IJavaSearchScope scope = SearchEngine
				.createJavaSearchScope(new IJavaElement[] { iProject });
		SearchEngine searchEngine = new SearchEngine();
		try {
			CFSearchCallback test = new CFSearchCallback(searchFor, iProject);
			searchEngine.search(pattern, new SearchParticipant[] { SearchEngine
					.getDefaultSearchParticipant() }, scope, test, null);
		} catch (CoreException e) {
			Log.logError("CF error", e);
		}

		return false;

	}

	/**
	 * Sprawdzenie czy obecny typ jest type nadrzdnym wobec szukanego typu
	 * @param currentType - biecy typ 
	 * @param searchType - szukany typ
	 * @param proj - projekt
	 * @return warto okrelajc czy biecy typ jest nadrzdny
	 * @throws JavaModelException
	 */
	public boolean isSuperClass(IType currentType, IType searchType,
			IJavaProject proj) throws JavaModelException {

		ITypeHierarchy hierarchy = currentType.newTypeHierarchy(proj, null);

		IType[] supeClasses = hierarchy.getAllSuperclasses(currentType);

		for (int i = 0; i < supeClasses.length; i++) {

			if (supeClasses[i].equals(searchType)) {
				return true;
			}
		}

		return false;

	}

}
