package pl.put.poznan.metrics.calculators.mood;

import java.util.Iterator;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;

public class MethodHidingFactor extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na MHF - Method Hiding
	 * Factor
	 * 
	 */
	public MethodHidingFactor() {
		super(MHF);
		Logger.logMessage("MHF - konstruktor");
	}

	/**
	 * Obliczenie metryki MHF. Metrka jest obliczana na poziomie Projektu
	 * 
	 * @param AbstractMetricSource -
	 *            projekt na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>PROJECT</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double hideMethods = 0;
		double allMethods = 0;

		if (source.getLevel() != PROJECT)
			throw new InvalidSourceException(
					"Attribute Hiding Factor only applicable to project");

		IJavaProject iProject = (IJavaProject) source.getJavaElement();

		try {

			Iterator iter = Util.getAllTypes(iProject).iterator();

			while (iter.hasNext()) {
				IType iType = (IType) iter.next();

				if (!iType.isClass())
					continue;

				// allMethods+=iTypes[k].getMethods().length;
				allMethods += Util.countMethods(iType);
				hideMethods += getCountHiddenMethods(iType);
			}

			double result = 100d;

			Logger.logMessage("MHF all = " + allMethods);
			Logger.logMessage("MHF hide = " + hideMethods);

			if (allMethods != 0) {
				result = (long) ((hideMethods / allMethods) * 100);
			}

			Logger.logMessage("MHF result = " + result);

			source.setValue(new Metric(MHF, result));

		} catch (JavaModelException e) {
			source.setValue(new Metric(MHF, 0));
		}

	}

	/**
	 * Oblicza ilo ukrytych metod - metody private lub protected
	 * @param iType - typ na ktym wykonywane s oblczenia
	 * @return int - ilo ukrytych metod
	 * @throws JavaModelException
	 */
	public static int getCountHiddenMethods(IType iType)
			throws JavaModelException {

		int hiddenMethods = 0;

		IMethod[] iMethods = iType.getMethods();

		for (int index = 0; index < iMethods.length; index++) {

			if (((iMethods[index].getFlags() & Flags.AccPrivate) != 0)
					|| ((iMethods[index].getFlags() & Flags.AccProtected) != 0)) {

				hiddenMethods++;

			}
		}

		return hiddenMethods;

	}

}
