package pl.put.poznan.metrics.calculators.mood;

import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

public class MethodHidingFactorPerClass extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na MHF_PER_CLASS - Method Hiding Factor
	 * na poziomie pojedynczego typu
	 * 
	 */
	public MethodHidingFactorPerClass() {
		super(MHF_PER_CLASS);
		Logger.logMessage("MHF_PER_CLASS - konstruktor");
	}

	/**
	 * Obliczenie metryki MHF_PER_CLASS. Metrka jest obliczana na poziomie typw
	 * 
	 * @param AbstractMetricSource -
	 *            typ na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>TYPE</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double hideMethods = 0;
		double allMethods = 0;

		if (source.getLevel() != TYPE)
			throw new InvalidSourceException(
					"Attribute Hiding Factor only applicable to types");

		IType iType = (IType) source.getJavaElement();

		try {

			// allMethods+=iType.getMethods().length;
			allMethods += Util.countMethods(iType);
			hideMethods += MethodHidingFactor.getCountHiddenMethods(iType);

			double result = 100d;

			Logger.logMessage("MHF_PER_CLASS all = " + allMethods);
			Logger.logMessage("MHF_PER_CLASS hide = " + hideMethods);

			if (allMethods != 0) {
				result = (long) ((hideMethods / allMethods) * 100);
			}

			Logger.logMessage("MHF_PER_CLASS result = " + result);
			source.setValue(new Metric(MHF_PER_CLASS, result));

		} catch (JavaModelException e) {
			source.setValue(new Metric(MHF_PER_CLASS, 0));
		}

	}

}
