package pl.put.poznan.metrics.calculators.mood;

import java.util.Iterator;
import java.util.Set;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;

public class MethodInheritanceFactor extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na MIF - Method
	 * Inheritance Factor
	 * 
	 */
	public MethodInheritanceFactor() {
		super(MIF);
		Logger.logMessage("MIF - konstruktor");
	}

	/**
	 * Obliczenie metryki MIF. Metrka jest obliczana na poziomie Projektu
	 * 
	 * @param AbstractMetricSource -
	 *            projekt na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>PROJECT</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double inheritedMethods = 0;
		double allMethods = 0;

		if (source.getLevel() != PROJECT)
			throw new InvalidSourceException(
					"Method Inheritance Factor only applicable to project");

		IJavaProject iProject = (IJavaProject) source.getJavaElement();

		try {

			IPackageFragment[] iPackagesRoot = iProject.getPackageFragments();

			for (int i = 0; i < iPackagesRoot.length; i++) {

				ICompilationUnit[] iCompilationUnits = iPackagesRoot[i]
						.getCompilationUnits();

				for (int j = 0; j < iCompilationUnits.length; j++) {

					IType[] iTypes = iCompilationUnits[j].getAllTypes();

					for (int k = 0; k < iTypes.length; k++) {

						if (!iTypes[k].isClass())
							continue;

						int inhTemp = 0;

						Set superClassesMethodSignatures = Util
								.getSuperClassesMethodsSignatures(iTypes[k],
										iProject);

						// all attributes in this class
						// allMethods+=iTypes[k].getMethods().length;
						allMethods += Util.countMethods(iTypes[k]);

						// number of inherited methods
						inhTemp = getCountInherittedMethods(iTypes[k],
								superClassesMethodSignatures);
						inheritedMethods += inhTemp;

						// all attreibutes plus inherited attriubutes
						allMethods += inhTemp;

					}

				}

			}

			double result = 100d;

			Logger.logMessage("MIF, allMethods = " + allMethods);
			Logger.logMessage("MIF, inheritedMethods = " + inheritedMethods);

			if (allMethods != 0) {
				result = (long) ((inheritedMethods / allMethods) * 100);
			}

			Logger.logMessage("MIF, result = " + result);

			source.setValue(new Metric(MIF, result));

		} catch (JavaModelException e) {

			Logger.logError("MIF exception = ", e);
			source.setValue(new Metric(MIF, 0));

		}

	}

	/**
	 * Oblicza odziedziczone metody
	 * @param iType
	 * @param superClassMethodsSignatures
	 * @return ilo odziedziczonych metod
	 * @throws JavaModelException
	 */
	public static int getCountInherittedMethods(IType iType,
			Set superClassMethodsSignatures) throws JavaModelException {
		int inheritedMethods = superClassMethodsSignatures.size();

		IMethod[] iMethods = iType.getMethods();

		for (int index = 0; index < iMethods.length; index++) {

			Iterator iter = superClassMethodsSignatures.iterator();

			while (iter.hasNext()) {

				IMethod superMethod = (IMethod) iter.next();

				if (iMethods[index].isSimilar(superMethod)) {
					inheritedMethods--;
				}
			}

		}

		return inheritedMethods;

	}

}
