package pl.put.poznan.metrics.calculators.mood;

import java.util.Set;

import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;
import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;

public class MethodInheritanceFactorPerClass extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na MIF_PER_CLASS - Method Inheritance Factor
	 *  na poziomie pojedynczego typu
	 * 
	 */
	public MethodInheritanceFactorPerClass() {
		super(MIF_PER_CLASS);
		Logger.logMessage("MIF_PER_CLASS - konstruktor");
	}

	/**
	 * Obliczenie metryki MIF_PER_CLASS. Metrka jest obliczana na poziomie typw
	 * 
	 * @param AbstractMetricSource -
	 *            typ na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>TYPE</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double inheritedMethods = 0;
		double allMethods = 0;

		if (source.getLevel() != TYPE)
			throw new InvalidSourceException(
					"Method Inheritance Factor only applicable to type");

		IType iType = (IType) source.getJavaElement();

		try {
			Set superClassesMethodSignatures = Util
					.getSuperClassesMethodsSignatures(iType, iType
							.getJavaProject());

			// all attributes in this class
			// allMethods+=iType.getMethods().length;
			allMethods += Util.countMethods(iType);
			Logger.logMessage("MIF_CL, newMethods = "
					+ iType.getMethods().length);
			// number of inherited methods
			inheritedMethods += MethodInheritanceFactor
					.getCountInherittedMethods(iType,
							superClassesMethodSignatures);

			// all attreibutes plus inherited attriubutes
			allMethods += inheritedMethods;

			double result = 100d;

			Logger.logMessage("MIF_CL, type = " + iType.getElementName());
			Logger.logMessage("MIF_CL, allMethods = " + allMethods);
			Logger.logMessage("MIF_CL, inheritedMethods = " + inheritedMethods);

			if (allMethods != 0) {
				result = (long) ((inheritedMethods / allMethods) * 100);
			}

			Logger.logMessage("MIF_CL, result = " + result);

			source.setValue(new Metric(MIF_PER_CLASS, result));

		} catch (JavaModelException e) {

			Logger.logError("MIF_CL exception = ", e);
			source.setValue(new Metric(MIF_PER_CLASS, 0));

		}

	}

}
