package pl.put.poznan.metrics.calculators.mood;

import java.util.Iterator;
import java.util.Set;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;
import pl.put.poznan.metrics.util.ExtensionsConstants;
import pl.put.poznan.metrics.util.Logger;
import pl.put.poznan.metrics.util.Util;

public class PolymorphismFactor extends Calculator implements
		ExtensionsConstants {

	/**
	 * Kondtruktor bezparametrowy ustawiajcy typ metryki na PF - Polymorphism
	 * Factor
	 * 
	 */
	public PolymorphismFactor() {
		super(PF);
		Logger.logMessage("PF - konstruktor");
	}

	/**
	 * Obliczenie metryki PF. Metrka jest obliczana na poziomie Projektu
	 * 
	 * @param AbstractMetricSource -
	 *            projekt na ktrym ma zosta obliczona metryka
	 * @throws InvalidSourceException -
	 *             rzucany w przypadku podania do metody elemntu innego niz
	 *             <code>PROJECT</code>
	 */
	public void calculate(AbstractMetricSource source)
			throws InvalidSourceException {

		double newMethods = 0;
		double overridenMethods = 0;

		double denominator = 0;

		if (source.getLevel() != PROJECT)
			throw new InvalidSourceException(
					"Polymorphism Factor only applicable to project");

		IJavaProject iProject = (IJavaProject) source.getJavaElement();

		try {

			Iterator iter = Util.getAllTypes(iProject).iterator();

			while (iter.hasNext()) {
				IType iType = (IType) iter.next();

				if (!iType.isClass())
					continue;

				int overTemp = countOverridenMethods(iType, iProject);

				Logger.logMessage("PF, type = " + iType.getElementName()
						+ ", overriden = " + overTemp);

				overridenMethods += overTemp;
				// newMethods=iTypes[k].getMethods().length - overTemp;
				newMethods = Util.countMethods(iType) - overTemp;

				denominator += newMethods
						* Util.countDescendants(iType, iProject);

			}

			double result = 0d;

			if (0 != denominator) {
				result = (long) ((overridenMethods / denominator) * 100d);
			}

			Logger.logMessage("PF denominator = " + denominator);
			Logger.logMessage("PF overriden = " + overridenMethods);

			Logger.logMessage("PF result = " + result);

			source.setValue(new Metric(PF, result));

		} catch (JavaModelException e) {
			Logger.logError("PF exception", e);
			source.setValue(new Metric(PF, 0));
		}
	}

	/**
	 * Oblicza przesonite metody
	 * @param currentType - bieacy typ
	 * @param project - projekt
	 * @return int - liczba przesonitych metod
	 * @throws JavaModelException
	 */
	private int countOverridenMethods(IType currentType, IJavaProject project)
			throws JavaModelException {
		int overr = 0;
		Set superClassesMethods = Util.getSuperClassesMethodsSignatures(
				currentType, project);

		IMethod[] methods = currentType.getMethods();

		for (int i = 0; i < methods.length; i++) {

			Iterator iter = superClassesMethods.iterator();

			while (iter.hasNext()) {

				IMethod superMethod = (IMethod) iter.next();

				if ((superMethod.getFlags() & Flags.AccPrivate) != 0)
					continue;
				if ((superMethod.getFlags() & Flags.AccStatic) != 0)
					continue;

				if (methods[i].isSimilar(superMethod)) {
					overr++;
				}
			}

		}

		return overr;
	}

}
