package pl.put.poznan.metrics.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sourceforge.metrics.core.Log;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;

import pl.put.poznan.metrics.calculators.mood.CFSearchCallback;
import poznan.students.utils.CalculatorCache;

/**
 * Klasa narzdziowa zawierajca szereg metod statycznych, a take cache
 * aplikajci
 * 
 * @author Marek Bagiski
 * 
 */

public class Util {

	/**
	 * Zawiera cache aplikacji
	 */

	private static Map projectCache = new HashMap();

	// private static Map methodsSigantures = new HashMap();
	// private static List typeList = new ArrayList();
	// private static String projectName;

	// po zmianach w projekcie czyscimy cache
	static {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IResourceChangeListener listener = new IResourceChangeListener() {

			public void resourceChanged(IResourceChangeEvent arg0) {
				// System.out.println("RESOURCE CHANGED");
				clearCacheData();
			}

		};

		workspace.addResourceChangeListener(listener);
	}

	/**
	 * Zwraca sygantur podanej metody
	 * 
	 * @param method -
	 *            metoda, ktrej sygnatura jest pobierana
	 * @return String - sygnatura metody
	 * 
	 */
	public static String getMetodSiganture(IMethod method)
			throws JavaModelException {
		// - we remove return type from signature
		// String signature = new String (method.getElementName() + "_" +
		// method.getSignature().substring(0,method.getSignature().length() -
		// 1));
		String signature = new String(method.getElementName());

		String[] types = method.getParameterTypes();

		for (int i = 0; i < types.length; i++) {
			signature += "$" + types[i];
		}

		return signature;
	}

	/**
	 * 
	 * Pobiera metody (nieprywatne) ze wszystkich nadklas podanej klasy
	 * 
	 * @param iType -
	 *            klasa, proj - projekt return Set - zbir metoda
	 */
	public static Set getSuperClassesMethodsSignatures(IType iType,
			IJavaProject proj) throws JavaModelException {

		CalculatorCache calcCache = getProjectCache(proj.getElementName());

		String key = iType.getFullyQualifiedName();

		Object methods = calcCache.getMethodsSigantures().get(key);

		if (null != methods) {
			return (Set) methods;
		} else {

			Set superClassesMethods = new HashSet();

			ITypeHierarchy hierarchy = iType.newTypeHierarchy(proj, null);

			IType[] superClasses = hierarchy.getAllSuperclasses(iType);

			for (int i = 0; i < superClasses.length; i++) {

				IMethod[] iMethods = superClasses[i].getMethods();

				// Logger.logMessage("PF key = " + key);

				for (int j = 0; j < iMethods.length; j++) {

					// we don't want private methods
					if (((iMethods[j].getFlags() & Flags.AccPrivate) == 0)
							&& (!iMethods[j].isConstructor())) {
						superClassesMethods.add(iMethods[j]);
						// Logger.logMessage("XXX - dodalem metode = " +
						// iMethods[j].getElementName());
						iMethods[j].getReturnType();
					}

				}

			}

			calcCache.getMethodsSigantures().put(key, superClassesMethods);
			projectCache.put(proj.getElementName(), calcCache);
			return superClassesMethods;
		}

	}

	/**
	 * Zlicza potomkw danej klasy
	 * 
	 * @return int
	 * @param currentType -
	 *            klasa, proj - projekt
	 */
	public static int countDescendants(IType currentType, IJavaProject project)
			throws JavaModelException {

		ITypeHierarchy hierarchy = currentType.newTypeHierarchy(project, null);

		IType[] subClasses = hierarchy.getSubclasses(currentType);

		return subClasses.length;

	}

	/**
	 * Zlicza metody klasy ( nie uwzglednia konstruktorw )
	 * 
	 * @return int
	 * @param currentType -
	 *            klasa
	 * 
	 */
	public static int countMethods(IType currentType) throws JavaModelException {
		int numberOfMethods = 0;

		IMethod[] methods = currentType.getMethods();

		for (int i = 0; i < methods.length; i++) {
			if (!methods[i].isConstructor()) {
				numberOfMethods++;
			}
		}

		return numberOfMethods;
	}

	/**
	 * 
	 * Pobiera metody zgodne z podan wartoci flagi
	 * 
	 * @param iType -
	 *            klasa, proj - projekt return Set - zbir metoda
	 */
	public static Set getMethods(IType iType, int flag)
			throws JavaModelException {

		Set methods = new HashSet();

		IMethod[] iMethods = iType.getMethods();

		for (int j = 0; j < iMethods.length; j++) {

			if (((iMethods[j].getFlags() & flag) != 0)
					&& (!iMethods[j].isConstructor())) {
				methods.add(iMethods[j]);
			}

		}

		return methods;

	}

	/**
	 * 
	 * Pobiera wszystkie typy w projekcie
	 * 
	 * @param proj -
	 *            projekt return List - lista typw
	 */
	public static List getAllTypes(IJavaProject iProject)
			throws JavaModelException {

		CalculatorCache calcCache = getProjectCache(iProject.getElementName());

		if (calcCache.getTypeList().size() == 0) {

			IPackageFragment[] iPackagesRoot = iProject.getPackageFragments();

			for (int i = 0; i < iPackagesRoot.length; i++) {

				ICompilationUnit[] iCompilationUnits = iPackagesRoot[i]
						.getCompilationUnits();

				for (int j = 0; j < iCompilationUnits.length; j++) {

					IType[] iTypes = iCompilationUnits[j].getAllTypes();

					for (int k = 0; k < iTypes.length; k++) {

						calcCache.getTypeList().add(iTypes[k]);
					}

				}

			}

			projectCache.put(iProject.getElementName(), calcCache);

		}

		return calcCache.getTypeList();

	}

	/**
	 * Czysci cache
	 * 
	 */
	public static void clearCacheData() {
		Logger.logMessage("Czyszczenie danych tymczasowych");
		// methodsSigantures.clear();
		// typeList.clear();
		projectCache.clear();
	}

	/**
	 * 
	 * Pobiera dane z cache dla podanego projektu
	 * 
	 * @param projectName -
	 *            nazwa projektu dla ktorego pobieramy cache
	 */
	public static CalculatorCache getProjectCache(String projectName) {
		Object prCache = projectCache.get(projectName);

		if (null == prCache) {
			prCache = new CalculatorCache();
		}

		CalculatorCache calcCache = (CalculatorCache) prCache;

		return calcCache;
	}

	public static Map getProjectCache() {
		return projectCache;
	}

	/**
	 * 
	 * Oblicza powiazania pomiedzy klasami ( dla metryk CF oraz CBO ). Pobrane
	 * dane zapisuje w pamieci cache
	 * 
	 * @param iProject -
	 *            projekt dla ktorego jest obliczane powiazeni pomiedzy klasami
	 */
	public static void countCoupling(IJavaProject iProject)
			throws JavaModelException {

		if (getProjectCache(iProject.getElementName()).getCouplingMap().size() != 0)
			return;

		Iterator iter = getAllTypes(iProject).iterator();

		int currentTypeNumber = 1;
		int totalCount = getAllTypes(iProject).size();
		while (iter.hasNext()) {
			IType iType = (IType) iter.next();

			Logger.logMessage("CF & CBO, working on type = "
					+ iType.getElementName() + ", licznik = "
					+ currentTypeNumber + "/" + totalCount);

			search(iProject, iType);

			currentTypeNumber++;
		}
	}

	/**
	 * Poszukuje podanego typu w projekcie
	 * 
	 * @param iProject -
	 *            projekt w ktorym szukamy, searchFor - poszukiwany typ
	 * 
	 */
	public static boolean search(IJavaProject iProject, IType searchFor) {
		SearchPattern pattern = SearchPattern.createPattern(searchFor,
				IJavaSearchConstants.REFERENCES);
		IJavaSearchScope scope = SearchEngine
				.createJavaSearchScope(new IJavaElement[] { iProject });
		SearchEngine searchEngine = new SearchEngine();
		try {
			CFSearchCallback test = new CFSearchCallback(searchFor, iProject);
			searchEngine.search(pattern, new SearchParticipant[] { SearchEngine
					.getDefaultSearchParticipant() }, scope, test, null);
		} catch (CoreException e) {
			Log.logError("CF error", e);
		}

		return false;

	}

}
