package pl.put.poznan.metrics.test;

import junit.framework.TestCase;
import net.sourceforge.metrics.calculators.Calculator;
import net.sourceforge.metrics.calculators.InvalidSourceException;
import net.sourceforge.metrics.core.Metric;
import net.sourceforge.metrics.core.sources.AbstractMetricSource;
import net.sourceforge.metrics.core.sources.TypeMetrics;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import pl.put.poznan.metrics.calculators.mood.AttributeHidingFactorPerClass;
import pl.put.poznan.metrics.util.ExtensionsConstants;

public class AHFPerClassTest extends TestCase{

	AbstractMetricSource source100;
	AbstractMetricSource source50;
	AbstractMetricSource source0;
		
	protected void setUp() throws Exception {
		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject("Testowy");
		project.open(null);
		IJavaProject iJavaProject = JavaCore.create(project);
		iJavaProject.open(null);
		IPackageFragment[] packageFrag = iJavaProject.getPackageFragments();
		for (int i = 0; i < packageFrag.length; i++) {
			//System.out.println("nazwa pakietu = " + packageFrag[i].getElementName());
			if ( TestConsts.PACKAGE_NAME.equalsIgnoreCase(packageFrag[i].getElementName())){
				ICompilationUnit compilationUnit = packageFrag[i].getCompilationUnit("AHFTest100.java");
				IType type = compilationUnit.getType("AHFTest100");
				source100 = new TypeMetrics();
				source100.setJavaElement(type);
				
				ICompilationUnit compilationUnit50 = packageFrag[i].getCompilationUnit("AHFTest50.java");
				IType type50 = compilationUnit50.getType("AHFTest50");
				source50 = new TypeMetrics();
				source50.setJavaElement(type50);
				
				ICompilationUnit compilationUnit0 = packageFrag[i].getCompilationUnit("AHFTest0.java");
				IType type0 = compilationUnit0.getType("AHFTest0");
				source0 = new TypeMetrics();
				source0.setJavaElement(type0);
			}
		}
	}

	public void testAHFPerClass100() throws JavaModelException, InvalidSourceException{
		
		Calculator calc = new AttributeHidingFactorPerClass();
		calc.calculate(source100);
		Metric metric = source100.getValue(ExtensionsConstants.AHF_PER_CLASS);
		assertEquals(100,metric.getValue(),0.1);
		
	}
	
	public void testAHFPerClass50() throws JavaModelException, InvalidSourceException{
		
		Calculator calc = new AttributeHidingFactorPerClass();
		calc.calculate(source50);
		Metric metric = source50.getValue(ExtensionsConstants.AHF_PER_CLASS);
		assertEquals(50,metric.getValue(),0.1);
		
	}

	public void testAHFPerClass0() throws JavaModelException, InvalidSourceException{
	
		Calculator calc = new AttributeHidingFactorPerClass();
		calc.calculate(source0);
		Metric metric = source0.getValue(ExtensionsConstants.AHF_PER_CLASS);
		assertEquals(0,metric.getValue(),0.1);
	
	}
	
	protected void tearDown() throws Exception {
		
	}

}
